// (c) Copyright 2011-2012 Cloudera, Inc. All rights reserved.
define([
], function() {
	/**
	 * depends on bootstrap-model.
	 * options {
	 *   defaultVisible: true|false, // whether the dialog should be shown initially.
	 *   destroyOnClose: true|false // whether closing the dialog should remove it from the DOM.
	 * }
	 */
	$.fn.ModalDialogBase = function(options) {
		return this.each(function() {
			var $this = $(this);
			var opts = jQuery.extend({}, jQuery.fn.ModalDialogBase.defaults, options);

			if (opts.defaultVisible) {
				$this.modal();
			} else {
				$this.modal("hide");
			}
			if (opts.destroyOnClose) {
				// Remove on hide.
				// When user closes the dialog, we actually want to remove
				// it from the DOM. This is necessary when dealing with
				// dialogs that are AJAX.
				$this.on('hidden', function(){
					// We cannot remove the dialog immediately because
					// sometimes, we want to submit the form on the popup
					// itself, but that gets called after this function is
					// executed. So we have to delay the remove.
					setTimeout(function () {
						$this.remove();
					}, 50);
				});
			}

			// Press enter routes to the first primary button.
			// When user presses the Enter key, we want to invoke
			// the click event of the first primary button.
			// TODO: make this optional.
			$this.keypress(function(evt) {
				var code = evt.keyCode || evt.which;
				// Prevent enter key from submitting the form.
				if (code === $.ui.keyCode.ENTER) {
					var $primaryButtons = $this.find(".modal-footer").find(".btn-primary:first");
					if ($primaryButtons.length > 0) {
						$primaryButtons.trigger('click');
						if (evt) {
							evt.preventDefault();
						}
					} else {
						var $buttons = $this.find(".modal-footer").find(".btn:first");
						if ($buttons.length > 0) {
							$buttons.trigger('click');
							if (evt) {
								evt.preventDefault();
							}
						}
					}
				}
			});
		});
	};
	jQuery.fn.ModalDialogBase.defaults = {
		defaultVisible: true,
		destroyOnClose: true
	};
	return $.fn.ModalDialogBase;
});
